//=========================================================
// src/Interrupts.c: generated by Hardware Configurator
//
// This file will be regenerated when saving a document.
// leave the sections inside the "$[...]" comment tags alone
// or they will be overwritten!
//=========================================================

// USER INCLUDES
#include <SI_EFM8UB2_Register_Enums.h>
#include "RGBSensorLEDFeedback_Defs.h"
#include "VCPXpress.h"
#include "descriptor.h"

unsigned char SFRPAGE_SAVE;

//state variables are initialized to IDLE
unsigned char SPI_State = IDLE;
unsigned char I2C_State = IDLE;

unsigned int RawTempData;
unsigned char TempDataTx[TEMP_DATA_LEN];
bit TEMP_DATA_READY = FALSE;

/*This array is filled with 0xFF for debugging purposes; you can quickly determine how
 * many bytes were actually received and stored in the array because unused bytes will
 * remain 0xFF.*/
unsigned char I2C_RcvData[10] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

unsigned char RcvdByteCount;	//this is used as the index for I2C_RcvData[]

//-----------------------------------------------------------------------------
// SPI0_ISR
//-----------------------------------------------------------------------------
//
// SPI0 ISR Content goes here. Remember to clear flag bits:
// SPI0CN0::MODF (Mode Fault Flag)
// SPI0CN0::RXOVRN (Receive Overrun Flag)
// SPI0CN0::SPIF (SPI# Interrupt Flag)
// SPI0CN0::WCOL (Write Collision Flag)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (SPI0_ISR, SPI0_IRQn)
{
	//SPI registers are on all SFR pages, so need need to modify SFRPAGE

	SPI0CN0 &= ~BIT7;	//clear interrupt flag

	switch(SPI_State)
	{
			//SPI communications with thermocouple IC======================================
		case FIRST_TC_BYTE_SENT:
			RawTempData = SPI0DAT;
			SPI0DAT = 0x00;	//write a dummy byte to generate the clocks needed for reading data
			SPI_State = SECOND_TC_BYTE_SENT;
			break;

		case SECOND_TC_BYTE_SENT:
			TC_NSS = HIGH;	//disable slave select

			RawTempData = (RawTempData << 8) | SPI0DAT;

			/* The following instructions convert the raw
			 * binary temperature data into a format that
			 * can be easily interpreted by Scilab.*/
			TempDataTx[0] = LOWBYTE(RawTempData >> 12);
			TempDataTx[1] = LOWBYTE(RawTempData >> 4);
			switch((RawTempData & (BIT3|BIT2)) >> 2)
			{
				case 0: TempDataTx[2] = 0;
				break;

				case 1: TempDataTx[2] = 25;
				break;

				case 2: TempDataTx[2] = 50;
				break;

				case 3: TempDataTx[2] = 75;
				break;
			}
			TEMP_DATA_READY = TRUE;
			SPI_State = IDLE;
			break;

			//SPI communications with DAC===========================================
		case FIRST_DAC_BYTE_SENT:
			SPI0DAT = UpdateDAC_SecondByte;
			SPI_State = SECOND_DAC_BYTE_SENT;
			break;

		case SECOND_DAC_BYTE_SENT:
			DAC_NSS = HIGH;	//disable slave select
			SPI_State = IDLE;
			break;
	}
}

//-----------------------------------------------------------------------------
// SMBUS0_ISR
//-----------------------------------------------------------------------------
//
// SMBUS0 ISR Content goes here. Remember to clear flag bits:
// SMB0CN0::SI (SMBus Interrupt Flag)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (SMBUS0_ISR, SMBUS0_IRQn)
{
	SFRPAGE_SAVE = SFRPAGE;
	SFRPAGE = SMB0_PAGE;

	switch(I2C_State)
	{
		//Master Read===================================================
				//start condition transmitted
		case MstR_STA_SENT:
			SMB0CN0_STA = 0;	//clear start-condition bit
			SMB0CN0_STO = 0;	//make sure that stop-condition bit is cleared
			SMB0DAT = (I2C_SlaveAddr<<1)|BIT0;	//combine slave address with R/nW = 1
			I2C_State = MstR_ADDR_SENT;		//set state variable to next state
			SMB0CN0_SI = 0;	//clear interrupt flag
			break;

				//master transmitted "address + R/W" byte
		case MstR_ADDR_SENT:
			if(SMB0CN0_ACK == I2C_NACK)	//if slave did not ACK
			{
				//cancel transmission and release bus, as follows:
				SMB0CN0_STO = 1;	//transmit stop condition
				I2C_State = IDLE;	//set current state as IDLE
			}
			else	//if slave ACKed
			{
				if(I2C_NumReadBytes == 1)	//if only one byte will be read
				{
					//master NACKs next byte to say "stop transmitting"
					SMB0CN0_ACK = I2C_NACK;
				}
				else	//if more than one byte will be read
				{
					//master ACKs next byte to say "continue transmitting"
					SMB0CN0_ACK = I2C_ACK;
				}
				RcvdByteCount = 0;	//this variable will be an index for storing received bytes in an array
				I2C_State = MstR_READ_BYTE;	//set next state
			}
			SMB0CN0_SI = 0;	//clear interrupt flag
			break;

				//master received a byte
		case MstR_READ_BYTE:
			I2C_RcvData[RcvdByteCount] = SMB0DAT;	//store received byte
			RcvdByteCount++;	//increment byte counter (which is also the array index)
			SMB0CN0_SI = 0;	//clear interrupt flag

			if(RcvdByteCount == I2C_NumReadBytes)	//if this was the final byte
			{
				//release bus, as follows:
				SMB0CN0_STO = 1;	//transmit stop condition
				SMB0CN0_SI = 0;	//clear interrupt flag
				I2C_State = MstR_DATA_READY;	//this state tells the while loop in main() that the received data is ready
			}
			else if(RcvdByteCount == (I2C_NumReadBytes-1))	//if the next byte is the final byte
			{
				SMB0CN0_ACK = I2C_NACK;	//master NACKs next byte to say "stop transmitting"
			}
			else
			{
				SMB0CN0_ACK = I2C_ACK;	//master ACKs next byte to say "continue transmitting"
			}
			break;

		//Master Write===================================================
			//start condition transmitted
		case MstW_STA_SENT:
			SMB0CN0_STA = 0;	//clear start-condition bit
			SMB0CN0_STO = 0;	//make sure that stop-condition bit is cleared
			SMB0DAT = (I2C_SlaveAddr<<1);	//combine slave address with R/nW = 0
			I2C_State = MstW_ADDR_SENT;	//set state variable to next state
			SMB0CN0_SI = 0;	//clear interrupt flag
			break;

			//master transmitted "address + R/W" byte
		case MstW_ADDR_SENT:
			if(SMB0CN0_ACK == I2C_NACK)	//if slave did not ACK
			{
				//cancel transmission and release bus, as follows:
				SMB0CN0_STO = 1;	//transmit stop condition
				I2C_State = IDLE;	//set current state as IDLE
			}
			else	//if slave ACKed
			{
				SMB0DAT = *I2C_WriteBufferPtr;	//write first byte to SMBus data register
				I2C_State = MstW_BYTE_SENT;	//set next state
			}
			SMB0CN0_SI = 0;	//clear interrupt flag
			break;

			//master transmitted a byte
		case MstW_BYTE_SENT:
			if(SMB0CN0_ACK == I2C_NACK)	//if slave NACKed
			{
				//stop transmission and release bus, as follows:
				SMB0CN0_STO = 1;	//transmit stop condition
				I2C_State = IDLE;	//set current state as IDLE
			}
			//if slave ACKed and this was the final byte
			else if(I2C_WriteBufferPtr == I2C_FinalWriteAddress)
			{
				SMB0CN0_STO = 1;	//transmit stop condition
				I2C_State = IDLE;	//set current state as IDLE
			}
			//if slave ACKed and this was not the final byte
			else
			{
				I2C_WriteBufferPtr++;	//increment pointer that points at data to be transmitted
				SMB0DAT = *I2C_WriteBufferPtr;	//write next byte to SMBus data register
			}
			SMB0CN0_SI = 0;	//clear interrupt flag
			break;
	}

	SFRPAGE = SFRPAGE_SAVE;
}

