//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8UB2_Register_Enums.h>                  // SFR declarations
#include "InitDevice.h"
#include "VCPXpress.h"
#include "descriptor.h"
#include "RGBSensorLEDFeedback_Defs.h"

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main(void) {

	unsigned char R_scaled, G_scaled, B_scaled;
	unsigned int R_word, G_word, B_word;
	float R_intensity, G_intensity, B_intensity, MaxIntensity;

	// Call hardware initialization routine
	enter_DefaultMode_from_RESET();

	//VCPXpress initialization
	//USB_Init(&InitStruct);

	//enable VCPXpress API interrupts
	//API_Callback_Enable(myAPICallback);

	//enable global interrupts
	IE_EA = 1;

	//tell the RGBC sensor to start performing measurements
	I2C_MasterWrite(RGBC_Tx_EnableConv);

	while (1)
	{
		Delay_10ms(100);	//the LED is updated once per second

		//load the proper register address (for reading) into the RGBC sensor
		I2C_MasterWrite(RGBC_Tx_SetReadRGBC);

		//read the RGBC data
		I2C_MasterRead(RGBC_Rx_RGBC);

		//wait until the read transaction is complete
		while(I2C_State != MstR_DATA_READY);
		I2C_State = IDLE;

		//extract the 16-bit R, G, and B values from the received data
		R_word = (I2C_RcvData[1] << 8) | I2C_RcvData[0];
		G_word = (I2C_RcvData[3] << 8) | I2C_RcvData[2];
		B_word = (I2C_RcvData[5] << 8) | I2C_RcvData[4];

		//apply correction factors based on the relative sensitivity of the three photodetectors
		R_intensity = R_word * 1.39;
		G_intensity = G_word * 1;
		B_intensity = B_word * 1.79;

		//determine which intensity is the highest of the three
		if(R_intensity >= G_intensity && R_intensity >= B_intensity)
			MaxIntensity = R_intensity;
		else if(G_intensity >= R_intensity && G_intensity >= B_intensity)
			MaxIntensity = G_intensity;
		else
			MaxIntensity = B_intensity;

		/*Now we scale each measurement into the range 0 to 100.
		 * This preserves the relative ratios of the three
		 * intensities but standardizes the absolute intensities.
		 * Thus, the appearance of the LED module is
		 * determined by the proportion of red, green,
		 * and blue light, not by the overall intensity of
		 * the incident light.*/
		R_scaled = (R_intensity/MaxIntensity)*100;
		G_scaled = (G_intensity/MaxIntensity)*100;
		B_scaled = (B_intensity/MaxIntensity)*100;

		/* This can be used to turn off the LED if the measured
		 * light intensity is too low. This helps to avoid distracting
		 * color changes related to irrelevant RGB variations detected
		 * during low-intensity lighting conditions. This functionality
		 * was used with the "Christmas lights" demonstration to make
		 * the LED turn off when the sensor was not illuminated by one
		 * of the lights.*/
		if(MaxIntensity < 100)
		{
			R_scaled = 0;
			G_scaled = 0;
			B_scaled = 0;
		}

		//update each DAC channel with the scaled values
		UpdateDAC(DAC_RGB_R, R_scaled);
		UpdateDAC(DAC_RGB_G, G_scaled);
		UpdateDAC(DAC_RGB_B, B_scaled);
	}
}
